@@ -9,16 +9,16 @@ function(x)
   paste('"', x, '"', sep = "")
 
 setGeneric("toJSON",
-function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L)
+function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0)  
   standardGeneric("toJSON"))
 
 setMethod("toJSON", "NULL",
-           function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              if(container) "[ null ] " else "null"
            })
 
 setMethod("toJSON", "ANY",
-           function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1  || length(names(x)) > 0, collapse = "\n", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
 
              if(isS4(x)) {
                paste("{", paste(dQuote(slotNames(x)), sapply(slotNames(x), function(id) toJSON(slot(x, id), ...)), sep = ": "),
@@ -31,10 +31,10 @@ setMethod("toJSON", "ANY",
 
 
 setMethod("toJSON", "integer",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 1, collapse = "\n  ", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 1, collapse = "\n  ", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
 
              if(container) {
-                if(length(names(x)))
+                if(.withNames)
                    paste(sprintf("{%s", collapse), paste(dQuote(names(x)), x, sep = ": ", collapse = sprintf(",%s", collapse)), sprintf("%s}", collapse))
                 else
                    paste("[", paste(x, collapse = ", "), "]")
@@ -46,10 +46,10 @@ setMethod("toJSON", "integer",
 setOldClass("hexmode")
 
 setMethod("toJSON", "hexmode",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n   ", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n   ", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              tmp = paste("0x", format(x), sep = "")
              if(container) {
-                if(length(names(x)))
+                if(.withNames)
                    paste(sprintf("{%s", collapse), paste(dQuote(names(x)), tmp, sep = ": ", collapse = sprintf(",%s", collapse)), sprintf("%s}", collapse))
                 else               
                 paste("[", paste(tmp, collapse = ", "), "]")
@@ -59,7 +59,7 @@ setMethod("toJSON", "hexmode",
 
 
 setMethod("toJSON", "factor",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              toJSON(as.character(x), container, collapse, ..., .level = .level)
            })
 
@@ -67,7 +67,7 @@ setMethod("toJSON", "logical",
            function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
              tmp = ifelse(x, "true", "false")
              if(container) {
-                if(length(names(x)))
+                if(.withNames)
                    paste(sprintf("{%s", collapse), paste(dQuote(names(x)), tmp, sep = ": ", collapse = sprintf(",%s", collapse)), sprintf("%s}", collapse))
                 else               
                    paste("[", paste(tmp, collapse = ", "), "]")
@@ -76,10 +76,10 @@ setMethod("toJSON", "logical",
            })
 
 setMethod("toJSON", "numeric",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              tmp = formatC(x, digits = digits)
              if(container) {
-                if(length(names(x)))
+                if(.withNames)
                    paste(sprintf("{%s", collapse), paste(dQuote(names(x)), tmp, sep = ": ", collapse = sprintf(",%s", collapse)),
                                    sprintf("%s}", collapse))
                 else
@@ -90,13 +90,13 @@ setMethod("toJSON", "numeric",
 
 
 setMethod("toJSON", "character",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", digits = 5, ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              tmp = gsub("\\\n", "\\\\n", x)
              tmp = gsub('"', '\\\\"', tmp)
              tmp = gsub('(\\\\)', '\\1\\1', tmp)                          
              tmp = dQuote(tmp)
              if(container) {
-                if(length(names(x)))
+                if(.withNames)
                    paste(sprintf("{%s", collapse),
                           paste(dQuote(names(x)), tmp, sep = ": ", collapse = sprintf(",%s", collapse)),
                          sprintf("%s}", collapse))
@@ -110,31 +110,32 @@ setMethod("toJSON", "character",
 
 # Symbols.
 setMethod("toJSON", "name",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              as.character(x)
            })
 
 setMethod("toJSON", "AsIs",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...) {
+           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level=1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
               toJSON(structure(x, class = class(x)[-1]), container = TRUE, collapse = collapse, ...)
            })
 
 
 
 setMethod("toJSON", "matrix",
-           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...) {
+           function(x, container = length(x) > 1 || length(names(x)) > 0, collapse = "\n", ...,
+                    .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
              tmp = paste(apply(x, 1, toJSON), collapse = sprintf(",%s", collapse))
              if(!container)
                return(tmp)
 
-              if(length(names(x)))
+              if(.withNames)
                 paste("{", paste(dQuote(names(x)), tmp, sep = ": "), "}")                
               else
                 paste("[", tmp, "]")
            })
 
 setMethod("toJSON", "list",
-           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L) {
+           function(x, container = .level == 1L || length(x) > 1 || length(names(x)) > 0, collapse = "\n", ..., .level = 1L, .withNames = length(x) > 0 && length(names(x)) > 0) {
                 # Degenerate case.
              if(length(x) == 0) {
                           # x = structure(list(), names = character()) gives {}
@@ -149,7 +150,7 @@ setMethod("toJSON", "list",
              if(!container)
                return(els)
              
-             if(length(names(x)))
+             if(.withNames)
                 paste(sprintf("{%s", collapse),
                       paste(dQuote(names(x)), els, sep = ": ", collapse = sprintf(",%s", collapse)),
                       sprintf("%s}", collapse))
